//
//  MyController.m
//  ImageCalc
//
//  Created by Cameron Hayne on Tue May 21 2002.
//  Copyright (c) 2002 Hayne of Tintagel. All rights reserved.
//

#import "MyController.h"
#import "MyModel.h"
#import "MyDataView.h"

@implementation MyController

- (id)init
{
    self = [super init];
    if (self == nil)
        return nil;

    model = [[MyModel alloc] init];
    return self;
}

- (void)dealloc
{
    [timer invalidate];
    [timer release];
    [model release];
    [super dealloc];
}

- (void)awakeFromNib
{
    // get the initial value of the model's parameter and send it to the slider
    [slider setFloatValue: [model parameter]];
    
    [self setupDrawer];
}

// The controller gets notified of window resizes since it is the window's delegate
- (void)windowDidResize:(NSNotification *)aNotification
{
    NSSize mSize;
    NSSize size = [dataView preferredSize];
    
    // ask the model to change its size
    mSize = [model requestSize: size];

    // tell the dataView to change to 'mSize' (the size that the model changed to)
    [dataView setSize: mSize];

    // reset the bookkeeping variables
    info.numMegaPoints = mSize.width * mSize.height / 1.0e6;
    info.startTime = [NSDate timeIntervalSinceReferenceDate];
    info.lastTime = info.startTime;
    info.lastModelUpdates = 0;
    info.lastViewUpdates = 0;
    info.numUpdates = 0;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    int fpsMax = 15;
    float timeout = 1.0 / fpsMax;

    // set up the model for the current window size
    [self windowDidResize: nil];
    
    // We schedule timers to fire every 'timeout' seconds
    // at which time we will tell the view to update itself
    timer = [[NSTimer scheduledTimerWithTimeInterval: timeout
                                              target: self
                                            selector: @selector(updateDataView:)
                                            userInfo: nil
                                             repeats: YES] retain];

    // We need the following line so that the view updates continue
    // when the user is dragging the slider
    [[NSRunLoop currentRunLoop] addTimer:timer
                                 forMode:NSEventTrackingRunLoopMode];

    [model startUpdate];
}

// the following method is called several times per second via the timer
- (void)updateDataView:(NSTimer *)theTimer
{
    // Tell the view to redraw itself if it isn't displaying the latest data
    if ([dataView dataVersion] != [model dataVersion])
    {
        [dataView setNeedsDisplay:YES];
    }

    // Calculate the update rates and display them in the drawer
    if (info.numUpdates % 5 == 0)  // no need to do this every time
    {
        [self updateStatistics];
    }
    info.numUpdates++;
}

- (void) updateStatistics
{
    NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
    NSTimeInterval totalElapsedSec = now - info.startTime;
    NSTimeInterval elapsedSec      = now - info.lastTime;
    int totalModelUpdates = [model dataVersion];
    int totalViewUpdates  = [dataView numRedraws];
    int modelUpdates = totalModelUpdates - info.lastModelUpdates;
    int viewUpdates = totalViewUpdates - info.lastViewUpdates;

    float modelFps     = modelUpdates / elapsedSec;
    float modelAvgFps  = totalModelUpdates / totalElapsedSec;
    float modelMpps    = modelFps * info.numMegaPoints;
    float modelAvgMpps = modelAvgFps * info.numMegaPoints;

    float viewFps      = viewUpdates / elapsedSec;
    float viewAvgFps   = totalViewUpdates / totalElapsedSec;
    float viewMpps     = viewFps * info.numMegaPoints;
    float viewAvgMpps  = viewAvgFps * info.numMegaPoints;

    [modelFpsLabel     setIntValue:(int)(modelFps + 0.5)];
    [modelAvgFpsLabel  setIntValue:(int)(modelAvgFps + 0.5)];
    [modelMppsLabel    setFloatValue:modelMpps];
    [modelAvgMppsLabel setFloatValue:modelAvgMpps];

    [viewFpsLabel      setIntValue:(int)(viewFps + 0.5)];
    [viewAvgFpsLabel   setIntValue:(int)(viewAvgFps + 0.5)];
    [viewMppsLabel     setFloatValue:viewMpps];
    [viewAvgMppsLabel  setFloatValue:viewAvgMpps];

    info.lastTime = now;
    info.lastModelUpdates = totalModelUpdates;
    info.lastViewUpdates = totalViewUpdates;
}

- (IBAction)sliderAction:(id)sender
{
    [model setParameter:[sender floatValue]];
}

- (void)setupDrawer
{
    // the drawer might get opened or closed via the menu item
    // or manually (pushing it closed with the moused)
    // so we make sure we are notified when the drawer is opened or closed
    // so that we can change the menu item label

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(drawerStateChanged:)
               name: NSDrawerDidCloseNotification
             object: drawer];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(drawerStateChanged:)
               name: NSDrawerDidOpenNotification
             object: drawer];
}

- (void)drawerStateChanged:(NSNotification *)notification
{
    NSDrawerState state = [drawer state];
    if (state == NSDrawerOpenState || state == NSDrawerOpeningState)
    {
        [drawerMenuItem setTitle:@"Hide Statistics"];
    }
    else
    {
        [drawerMenuItem setTitle:@"Show Statistics"];
    }
}

// action to open & close the drawer
- (IBAction)drawerButtonAction:(id)sender
{
    NSDrawerState state = [drawer state];
    if (state == NSDrawerOpenState || state == NSDrawerOpeningState)
    {
        [drawer close];
    }
    else
    {
        [drawer openOnEdge:NSMaxXEdge];
    }
}


// Accessors

- (NSData *)data
{
    return [model data];
}

- (int)dataVersion
{
    return [model dataVersion];
}

- (NSLock *)displayLock
{
    return [model displayLock];
}

@end
