//
//  MyDataView.m
//  ImageCalc
//
//  Created by Cameron Hayne on Tue May 21 2002.
//  Copyright (c) 2002 Hayne of Tintagel. All rights reserved.
//

#import "MyDataView.h"
#import "MyController.h"

@implementation MyDataView

static NSBitmapImageRep *createImageRep(int width, int height)
{
    NSBitmapImageRep *imageRep = [[NSBitmapImageRep alloc]
                      initWithBitmapDataPlanes: 0
                                    pixelsWide: width
                                    pixelsHigh: height
                                 bitsPerSample: 8
                               samplesPerPixel: 1
                                      hasAlpha: NO
                                      isPlanar: NO
                                colorSpaceName: NSDeviceWhiteColorSpace
                                   bytesPerRow: 0
                                  bitsPerPixel: 0];
    return imageRep;
}
    
- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self == nil)
        return nil;

    return self;
}

- (void)dealloc
{
    [imageRep release];
    [super dealloc];
}

- (void)awakeFromNib
{
    // We find out the offsets of the dataView relative to the window
    // so that we can use them to calculate an appropriate dataView frame
    // when the window is resized
    NSRect cvFrame = [[[self window] contentView] frame];
    NSRect dvFrame = [self frame];
    leftOffset = dvFrame.origin.x;
    rightOffset = cvFrame.size.width - (dvFrame.origin.x + dvFrame.size.width);
    topOffset = cvFrame.size.height - (dvFrame.origin.y + dvFrame.size.height);
    bottomOffset = dvFrame.origin.y;
}

// calculates the dataView's preferred size for the current window
- (NSSize)preferredSize
{
    NSSize cvSize = [[[self window] contentView] frame].size;
    NSSize size = NSMakeSize(cvSize.width - (leftOffset + rightOffset),
                             cvSize.height - (topOffset + bottomOffset));
    return size;
}

- (void)setSize:(NSSize)aSize
{
    NSRect frame;
    NSSize cvSize = [[[self window] contentView] frame].size; 

    // set the frame to be the given size
    frame.size = aSize;
    
    // and shift it so it remains centred
    frame.origin = NSMakePoint((leftOffset - rightOffset
                                + cvSize.width - aSize.width) / 2,
                               (bottomOffset - topOffset
                                + cvSize.height - aSize.height) / 2);

    // change the dataView's frame
    [self setFrame: frame];

    // tell the superview to redisplay in case there are any gaps due to the size change
    [[self superview] setNeedsDisplay:YES];

    // set the window title to indicate the model's width & height
    [[self window] setTitle:[NSString stringWithFormat:@"ImageCalc %g x %g",
                                                       aSize.width, aSize.height]];

    // create an ImageRep for the dataView
    [imageRep release];
    imageRep = createImageRep(aSize.width, aSize.height);
    imageBytesPerRow = [imageRep bytesPerRow]; // same as 'width' since 8 bits per pixel
    //NSLog(@"imageBytesPerRow=%d", imageBytesPerRow);

    // reset the bookkeeping variables
    dataVersion = -1;
    numRedraws = 0;
}

- (BOOL)isOpaque
{
    // this allows the graphics system to draw faster
    return YES;
}

static void drawStringInRect(NSRect rect, NSString *str, int fontSize)
{
    NSDictionary *dict = [NSDictionary
                          dictionaryWithObject: [NSFont boldSystemFontOfSize: fontSize]
                                        forKey: NSFontAttributeName];
    NSAttributedString *astr = [[NSAttributedString alloc]
                                initWithString: str
                                    attributes: dict];
    NSSize strSize = [astr size];
    NSPoint pt = NSMakePoint((rect.size.width - strSize.width) / 2,
                             (rect.size.height - strSize.height) / 2);

    // clear the rect
    rect.origin.x = 0;
    rect.origin.y = 0;
    [[NSColor whiteColor] set];
    NSRectFill(rect);

    // draw the string
    [astr drawAtPoint:pt];

    [astr release];
}

- (void)drawRect:(NSRect)aRect
{
    // we ignore the 'aRect' argument - possible future optimization would use it
    NSRect		frame = [self frame];
    NSData		*data;
    BOOL		imageRepInit;

    if ([self inLiveResize])
    {
        NSString *str = [NSString stringWithFormat: @"Resizing to %g x %g",
                                                    frame.size.width, frame.size.height];
        drawStringInRect(frame, str, 20);
        return;
    }
    
    // we use the model 'data' to fill in the 'bitmapData' before drawing

    [[controller displayLock] lock];       // ensure that 'data' doesn't get changed
    data = [controller data];
    dataVersion = [controller dataVersion];
    if (data == nil)
    {
        drawStringInRect(frame, @"Initializing...", 20);
        imageRepInit = NO;
    }
    else
    {
        const char	*dataBuffer = [data bytes];
        char		*imageBuffer = [imageRep bitmapData];
        int		height = frame.size.height;
        int 		row, col, index;

        // fill in the bytes of the imageRep using 'data'
        index = 0;
        for (row = 0; row < height; row++)
        {
            for (col = 0; col < imageBytesPerRow; col++)
            {
                imageBuffer[index] = dataBuffer[index];
                index++;
            }
        }
        imageRepInit = YES;
    }

    [[controller displayLock] unlock]; // allow the calculation to change 'data'

    if (imageRepInit)
    {
        [imageRep draw];
        numRedraws++;
    }
}

- (void)viewWillStartLiveResize
{
    [super viewWillStartLiveResize];
    // could do something here if needed
}

- (void)viewDidEndLiveResize
{
    [super viewDidEndLiveResize];
    // could do something here if needed
}


// Accessors

- (int)dataVersion
{
    return dataVersion;
}

- (int)numRedraws
{
    return numRedraws;
}

@end
