//
//  MyModel.m
//  ImageCalc
//
//  Created by Cameron Hayne on Tue May 21 2002.
//  Copyright (c) 2002 Hayne of Tintagel. All rights reserved.
//

#import "MyModel.h"

// let's assume that the model width must be a multiple of 32
static int widthIncr = 32;

@implementation MyModel

- (id)init
{
    self = [super init];
    if (self == nil)
        return nil;

    calculationLock = [[NSLock alloc] init];
    displayLock = [[NSLock alloc] init];
    parameter = 180.0;
    return self;
}

- (void)dealloc
{
    [calculationLock release];
    [displayLock release];
    [data release];
    [newData release];
    [super dealloc];
}

static void initData(int width, int height, char *dataBytes)
{
    int row, col;
    int index = 0;
    for (row = 0; row < height; row++)
    {
        for (col = 0; col < width; col++, index++)
        {
            dataBytes[index] = (col > row) ? 0xFF : 0x00;
        }
    }
}

// the following method is called from the controller when the window has been resized
- (NSSize)requestSize:(NSSize)aSize
{
    int numBytes;

    // ensure that the width is a multiple of 'widthIncr'
    int extraWidth = (int)aSize.width % widthIncr;
    aSize.width -= extraWidth;

    [calculationLock lock];    // make sure we're not in the middle of a calculation
    height = (int)aSize.height;
    width = (int)aSize.width;
    numBytes = height * width; // one byte per datum
    [data release];
    [newData release];
    data =    [[NSMutableData dataWithLength: numBytes] retain];
    newData = [[NSMutableData dataWithLength: numBytes] retain];
    initData(width, height, [data mutableBytes]);
    dataVersion = 0;
    [calculationLock unlock]; // allow the calculation thread to use 'data'

    return aSize;  // informs the sender of the model's size
}

- (void)startUpdate
{
    shouldContinueUpdating = TRUE;
    [NSThread detachNewThreadSelector: @selector(update:)
                             toTarget: self
                           withObject: nil];
}

- (void)stopUpdate
{
    shouldContinueUpdating = FALSE;
    // this makes the update method finish and so the thread exits
}

// calculates the new data values (based on 'dataBytes' & 'parameter')
// and puts the new values in 'newDataBytes'
static void calcData(int width, int height, float parameter,
                     const char *dataBytes, char *newDataBytes)
{
    // The value of 'parameter' is set by the slider.
    // We define a threshold value as a function of 'parameter'
    // that makes the slider have a perceptible effect
    int threshold = 250  + (int)(5.0/255.0 * parameter);
    int row, col;
    int index = 0;
    for (row = 0; row < height; row++)
    {
        for (col = 0; col < width; col++, index++)
        {
            int rnd = random() >> 23;  // ranges from 0 to 255
            if (rnd > threshold)
            {
                int mix = random() >> 23;
                newDataBytes[index] = (dataBytes[index] + mix) / 2;
            }
            else
            {
                newDataBytes[index] = dataBytes[index];
            }
        }
    }
}

// the following is executed in the thread started in the init method
- (void)update:(id)anObject
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        
    while (shouldContinueUpdating)
    {
        // The initialization of 'dataBytes' & 'newDataBytes'
        // must be inside the loop since 'data' & 'newData' might change
        // when the view is resized
        char *dataBytes, *newDataBytes;

        [calculationLock lock];       // ensure that 'data' doesn't get resized
        dataBytes = [data mutableBytes];
        newDataBytes = [newData mutableBytes];
        
        calcData(width, height, parameter, dataBytes, newDataBytes);
        
        [displayLock lock];           // ensure we aren't in the middle of a redraw
        [newData getBytes:dataBytes]; // copy newData into data
        dataVersion++;
        [displayLock unlock];         // allow redraws to use 'data'
        [calculationLock unlock];     // allow 'data' to be resized if needed
    }
    
    [pool release];
}


// Accessors

- (NSSize)size
{
    return NSMakeSize(width, height);
}

- (NSData *)data
{
    return data;	
}

- (int)dataVersion
{
    return dataVersion;
}

- (NSLock *)displayLock
{
    return displayLock;
}

- (float)parameter
{
    return parameter;
}

- (void)setParameter:(float)value
{
    parameter = value;
}

@end


